# MODULES
import json
import os
from pathlib import Path


def open_json_file(path: Path, encoding="utf-8"):
    if not os.path.exists(path):
        raise FileExistsError(f"File {path} does not exist")

    with open(path, encoding=encoding) as json_file:
        raw_data: dict = json.load(json_file)

    return raw_data



# MODULES
from enum import Enum
from typing import Dict, Optional, Tuple

# PYDANTIC
from pydantic import BaseModel

# SQLALCHEMY
from sqlalchemy.orm import InstrumentedAttribute, RelationshipProperty


def uppercase(items: list[str] | str):
    if items is None:
        return

    if isinstance(items, list):
        return [item.strip().upper() for item in items]
    if isinstance(items, set):
        return {item.strip().upper() for item in items}
    else:
        return items.strip().upper()


def create_model_column_enum(model, exclude: list[str] = []):
    return create_enum(
        enum_name=f"{model.__name__}Enum",
        values=sorted(
            [
                attr.key
                for attr in model.__dict__.values()
                if isinstance(attr, InstrumentedAttribute)
                and not isinstance(attr.property, RelationshipProperty)
                and attr.key not in exclude
            ]
        ),
    )


def create_enum(enum_name, values) -> Enum:
    enum_members = {}
    for value in values:
        enum_members[value] = value
    return Enum(enum_name, enum_members)


def is_field_in_model(
    model,
    field: str,
    mapper_alias: Dict[str, Optional[str]],
) -> bool:
    field_alias = mapper_alias.get(field)
    item = field_alias if field_alias is not None else field
    base_model_fields = [
        attr.key
        for attr in model.__dict__.values()
        if isinstance(attr, InstrumentedAttribute)
        and not isinstance(attr.property, RelationshipProperty)
    ]
    return item in base_model_fields


def get_mapper_enum(model, schema: BaseModel) -> Tuple[Dict[str, Optional[str]], Enum]:
    mapper_alias = {k: v.alias for k, v in schema.model_fields.items()}
    enum = create_enum(
        "CONFIG_CATEGORY_ENUM",
        [
            item
            for item in schema.model_fields.keys()
            if is_field_in_model(model, item, mapper_alias)
        ],
    )

    return mapper_alias, enum


def remove_t_from_date_str(date_str: str) -> str:
    return date_str.replace("T", " ")


def nonesorter(a):
    if not a:
        return ""
    return a



# MODULES
import sys
import logging
import unidecode
from logging.handlers import TimedRotatingFileHandler
from pathlib import Path
from typing import Optional


class AlphaLogger:
    def __init__(
        self,
        name: str,
        directory: str,
        level: int = logging.INFO,
        when: str = "midnight",
        interval: int = 1,
        backup_count: int = 10,
        file_name: Optional[str] = None,
    ):
        if file_name is None:
            file_name = name

        error_name = "errors"
        warning_name = "warnings"
        monitoring_name = "monitoring"

        directory_path = Path(directory)
        directory_path.mkdir(parents=True, exist_ok=True)

        formatter = logging.Formatter(
            "%(asctime)s - %(levelname)s - %(message)s", datefmt="%Y-%m-%d %H:%M:%S"
        )

        logger_config = {
            "level": level,
            "directory_path": directory_path,
            "when": when,
            "interval": interval,
            "backup_count": backup_count,
            "formatter": formatter,
        }

        self._logger = self._create_logger(
            name=name,
            file_name=file_name,
            stream_output=True,
            **logger_config,
        )
        self._error_logger = self._create_logger(
            name=error_name,
            file_name=error_name,
            **logger_config,
        )
        self._warning_logger = self._create_logger(
            name=warning_name,
            file_name=warning_name,
            **logger_config,
        )
        self._monitoring_logger = self._create_logger(
            name=monitoring_name,
            file_name=monitoring_name,
            **logger_config,
        )

    def info(
        self,
        message: str,
        exc_info=None,
        monitor: Optional[str] = None,
    ):
        self._logger.info(message, exc_info=exc_info)
        if monitor is not None:
            self._monitoring_logger.info(
                self._process_monitoring_message(message=message, monitor=monitor),
                exc_info=exc_info,
            )

    def warning(
        self,
        message,
        exc_info=None,
        monitor: Optional[str] = None,
    ):
        self._logger.warning(message, exc_info=exc_info)
        self._warning_logger.warning(message, exc_info=exc_info)
        if monitor is not None:
            self._monitoring_logger.info(
                self._process_monitoring_message(message=message, monitor=monitor),
                exc_info=exc_info,
            )

    def error(
        self,
        message,
        exc_info=None,
        monitor: Optional[str] = None,
    ):
        self._logger.error(message, exc_info=exc_info)
        self._error_logger.error(message, exc_info=exc_info)
        if monitor is not None:
            self._monitoring_logger.info(
                self._process_monitoring_message(message=message, monitor=monitor),
                exc_info=exc_info,
            )

    def critical(
        self,
        message,
        exc_info=None,
        monitor: Optional[str] = None,
    ):
        self._logger.critical(message, exc_info=exc_info)
        self._error_logger.critical(message, exc_info=exc_info)
        if monitor is not None:
            self._monitoring_logger.info(
                self._process_monitoring_message(message=message, monitor=monitor),
                exc_info=exc_info,
            )

    @classmethod
    def _process_monitoring_message(cls, message: str, monitor: str) -> str:
        return unidecode.unidecode(message.replace(message, f"[{monitor}] ({message})"))

    def _create_logger(
        self,
        name: str,
        level: int,
        directory_path: Path,
        file_name: str,
        when: str,
        interval: int,
        backup_count: int,
        formatter: logging.Formatter,
        stream_output: bool = False,
    ):
        logger = logging.getLogger(name=name)

        if logger.hasHandlers():
            return logger

        logger.setLevel(level)

        if stream_output:
            # Add a stream handler to log messages to stdout
            stream_handler = logging.StreamHandler(stream=sys.stdout)
            stream_handler.setLevel(level)
            stream_handler.setFormatter(formatter)
            logger.addHandler(stream_handler)

        # Add a file handler to log messages to a file
        time_rotating_handler = TimedRotatingFileHandler(
            filename=directory_path / f"{file_name}.log",
            when=when,
            interval=interval,
            backupCount=backup_count,
        )
        time_rotating_handler.setLevel(level)
        time_rotating_handler.setFormatter(formatter)
        logger.addHandler(time_rotating_handler)

        return logger



from setuptools import setup

version = "0.1.0"

setup(
    name="alphaz-next",
    version=version,
    packages=[
        "alphaz_next",
        "alphaz_next.libs",
        "alphaz_next.models",
        "alphaz_next.tests",
        "alphaz_next.utils",
    ],
    install_requires=[
        "sqlalchemy",
        "httpx",
        "python-jose",
        "dependency-injector",
        "python-multipart",
        "fastapi==0.100.0",
        "pydantic_settings",
        "pydantic==2.3",
        "sqlalchemy == 1.4.41",
        "sqlalchemy_utils",
        "pytz",
        "email-validator",
    ],
    license="MIT",
    author="Maxime MARTIN",
    author_email="maxime.martin02@hotmail.fr",
    description="A project to make a lib to start FASTAPI quickly",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/STDef200mm/alphaz-next",
    download_url="https://github.com/STDef200mm/alphaz-next/archive/refs/tags/%s.tar.gz"
    % version,
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
    ],
)
